import _extends from "@babel/runtime/helpers/esm/extends";
import { useContext, useReducer } from 'react';
import pick from 'object.pick';
import { createReducer, reducerHelper } from './reducer-utils';
import { initStateMetaHandler } from './init-state-meta-handler';
import { AppContext } from './app-context';
import { createStoreBindContext } from './create-store-bind-context';
import { createSharedContext } from './create-shared-context';
export function createStoreTakenMembraneHook(storeConfig, membraneStoreConfig) {
  if (!storeConfig.ref) {
    storeConfig.ref = {};
  }

  if (!membraneStoreConfig.ref) {
    membraneStoreConfig.ref = {};
  }

  var storeStateConfig = initStateMetaHandler(storeConfig);
  var membraneStoreStateConfig = initStateMetaHandler(membraneStoreConfig);
  var mergedStateKeys = [].concat(storeStateConfig.stateKeys, membraneStoreStateConfig.stateKeys);
  var reducer = reducerHelper(createReducer(mergedStateKeys));
  return function useMembraneStore(props) {
    var context = useContext(AppContext);

    var _useReducer = useReducer(reducer, _extends({}, storeStateConfig.initState, membraneStoreStateConfig.initState), function (initArgs) {
      return membraneStoreStateConfig.init(storeStateConfig.init(initArgs));
    }),
        state = _useReducer[0],
        dispatch = _useReducer[1];

    var mergedRef = {
      ref: _extends({}, storeConfig.ref, membraneStoreConfig.ref),
      refKeys: [].concat(storeStateConfig.refKeys, membraneStoreStateConfig.refKeys)
    };
    var sharedContext = createSharedContext(mergedStateKeys, dispatch, state, mergedRef);
    var superBindContext = createStoreBindContext(context, storeConfig, sharedContext);
    var finlayStore = createStoreBindContext(context, membraneStoreConfig, sharedContext, superBindContext, props);
    return Object.freeze({
      state: pick(finlayStore.state, storeStateConfig.stateKeys),
      refs: pick(finlayStore.refs, storeStateConfig.refKeys),
      controller: pick(finlayStore.controller, Object.keys(storeConfig.controller)),
      view: pick(finlayStore.view, Object.keys(storeConfig.view))
    });
  };
}